#!/usr/bin/env python
# encoding: utf-8

"""
@author: lqk
@contact: 798244092@qq.com
@site: https://github.com/lqkweb
@file: TimeFormat.py
@time: 2019/5/13 3:29 PM
"""
import re
import time
import datetime
import traceback
import copy

en_month_digital = {
    "Jan": "1",
    "Feb": "2",
    "Mar": "3",
    "Apr": "4",
    "May": "5",
    "Jun": "6",
    "Jul": "7",
    "Aug": "8",
    "Sep": "9",
    "Oct": "10",
    "Nov": "11",
    "Dec": "12",
    "JAN": "1",
    "FEB": "2",
    "MAR": "3",
    "APR": "4",
    "MAY": "5",
    "JUN": "6",
    "JUL": "7",
    "AUG": "8",
    "SEP": "9",
    "OCT": "10",
    "NOV": "11",
    "DEC": "12"
}


class TimeFormat(object):

    @staticmethod
    def mysql_url(dic):
        mysql_url_template = 'mysql+pymysql://{user}:{password}@{host}:{port}/{database}?charset={charset}'
        return mysql_url_template.TimeFormat(user=dic['user'], password=dic['password'], host=dic['host'], port=dic['port'],
                                         database=dic['database'], charset=dic['charset'])

    @staticmethod
    def redis_url(dic):
        redis_url_template = 'redis://:{password}@{host}:{port}/{db}'
        return redis_url_template.TimeFormat(password=dic['password'], host=dic['host'], port=dic['port'], db=dic['db'])

    @staticmethod
    def chinese_to_arabic_month(cn: str) -> str:
        check = {'一', '二', '三', '四', '五', '六', '七', '八', '九', '壹', '贰', '叁', '肆', '伍', '陆', '柒', '捌', '玖', '貮', '两',
                 '十', '百', '千', '万'}
        digit = {'一': 1, '二': 2, '三': 3, '四': 4, '五': 5, '六': 6, '七': 7, '八': 8, '九': 9,
                 '壹': 1, '贰': 2, '叁': 3, '肆': 4, '伍': 5, '陆': 6, '柒': 7, '捌': 8, '玖': 9, '貮': 2, '两': 2}

        def _trans(s):
            num = 0
            if s:
                idx_q, idx_b, idx_s = s.find('千'), s.find('百'), s.find('十')
                if idx_q != -1:
                    num += digit[s[idx_q - 1:idx_q]] * 1000
                if idx_b != -1:
                    num += digit[s[idx_b - 1:idx_b]] * 100
                if idx_s != -1:
                    # 十前忽略一的处理
                    num += digit.get(s[idx_s - 1:idx_s], 1) * 10
                if s[-1] in digit:
                    num += digit[s[-1]]
            return num

        def trans(chn):
            chn = chn.replace('零', '')
            idx_y, idx_w = chn.rfind('亿'), chn.rfind('万')
            if idx_w < idx_y:
                idx_w = -1
            num_y, num_w = 100000000, 10000
            if idx_y != -1 and idx_w != -1:
                return trans(chn[:idx_y]) * num_y + _trans(chn[idx_y + 1:idx_w]) * num_w + _trans(chn[idx_w + 1:])
            elif idx_y != -1:
                return trans(chn[:idx_y]) * num_y + _trans(chn[idx_y + 1:])
            elif idx_w != -1:
                return _trans(chn[:idx_w]) * num_w + _trans(chn[idx_w + 1:])
            return _trans(chn)

        cn_a = copy.deepcopy(cn)
        if (set(list(cn_a)) & check):
            cn_a = str(trans(cn_a))
        return cn_a

    @staticmethod
    def chinese_to_arabic(cn: str) -> int:
        # constants for chinese_to_arabic

        CN_NUM = {
            '〇': 0, '一': 1, '二': 2, '三': 3, '四': 4, '五': 5, '六': 6, '七': 7, '八': 8, '九': 9, '零': 0, '十': 1,
            '壹': 1, '贰': 2, '叁': 3, '肆': 4, '伍': 5, '陆': 6, '柒': 7, '捌': 8, '玖': 9, '貮': 2, '两': 2, '拾': 1,
        }

        CN_UNIT = {
            '十': 10,
            '拾': 10,
            '百': 100,
            '佰': 100,
            '千': 1000,
            '仟': 1000,
            '万': 10000,
            '萬': 10000,
            '亿': 100000000,
            '億': 100000000,
            '兆': 1000000000000,
        }

        cn_a = copy.deepcopy(cn)
        for i in cn:
            if CN_NUM.get(i) != None:
                cn_a = cn_a.replace(i, str(CN_NUM.get(i)))

        return cn_a

    @staticmethod
    def regex_date(str_date):
        expression = "(20\d{2}([\.\-/|年月\s]{1,3}\d{1,2}){2}日?(\s?\d{2}:\d{2}(:\d{2})?)?)|(\d{1,2}\s?(分钟|小时|天)前)"
        expression = re.compile(expression)
        date = expression.search(str_date)
        if date:
            return date.group().replace('/', '-').replace('.', '-') \
                .replace('年', '-').replace('月', '-').replace('日', '')

    @staticmethod
    def str_datetime(str_datetime):
        date_format = \
            [
                "%Y-%m-%d",
                "%Y%m%d",
                "%d %m, %Y",
                "%A, %B %d, %Y",
                "%a, %B %d, %Y",
                "%A, %b %d, %Y",
                "%a, %b %d, %Y",
                "%b %d, %Y",
                "%B %d, %Y",
                "%d %b, %Y",
                "%d %B, %Y",
                "%B %d %Y",
                "%d %b %Y",
                "%d %b %Y",
                "%d %B %Y",
                "%y-%m-%d"
            ]
        clock = \
            [
                "",
                "%H:%M:%S",
                "%H:%M",
                "%H"
            ]
        for i in date_format:
            for ii in clock:
                try:
                    df = "%s %s" % (i, ii)
                    return datetime.datetime.strptime(str_datetime.strip(), df.strip())
                except ValueError:
                    continue

    @staticmethod
    def time_normalization(date_time):
        format_check_en = "(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec|JAN|FEB|MAR|APR|MAY|JUN|JUL|AUG|SEP|OCT|NOV|DEC){1}"
        re_format_check_en = re.compile(format_check_en)
        check_en = re_format_check_en.search(date_time)
        if check_en and date_time:
            flag = True
            format_2_0 = ".*?(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec|JAN|FEB|MAR|APR|MAY|JUN|JUL|AUG|SEP|OCT|NOV|DEC){1}.*?(\d{1,2}).*?(\d{4})"
            format_2_1 = ".*?(\d{1,2}).*?(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec|JAN|FEB|MAR|APR|MAY|JUN|JUL|AUG|SEP|OCT|NOV|DEC){1}.*?(\d{4})"
            re_format_2_0 = re.compile(format_2_0)
            date_2_0 = re_format_2_0.match(date_time)
            re_format_2_1 = re.compile(format_2_1)
            date_2_1 = re_format_2_1.match(date_time)
            if date_2_0:
                date_cleaned_2_0 = date_2_0.group()
                month = date_2_0.group(1)
                day = date_2_0.group(2)
                year = date_2_0.group(3)
            elif date_2_1:
                date_cleaned_2_1 = date_2_1.group()
                day = date_2_1.group(1)
                month = date_2_1.group(2)
                year = date_2_1.group(3)
            else:
                flag = False
            if flag:
                date_cleaned_2 = '-'.join([year, en_month_digital[month], day])
                date_cleaned_2 = datetime.datetime.strptime(date_cleaned_2, "%Y-%m-%d")
                return date_cleaned_2
        if date_time:
            expression_1 = "(20\d{2}([\.\-/|年月\s]{0,3}\d{1,2}){2}日?(\s?\d{2}:\d{2}(:\d{2})?)?)"
            expression_1 = re.compile(expression_1)
            date_time_group_1 = expression_1.search(date_time)

            expression_2 = "({year}([\.\-/|年月\s]{0,3}\d{1,2}){2}日?(\s?\d{2}:\d{2}(:\d{2})?)?)".replace(
                '{year}', str(datetime.datetime.now().year)[-2:])
            expression_2 = re.compile(expression_2)
            date_time_group_2 = expression_2.search(date_time)
            number_head_base_regx = '([\u4e00\u4e8c\u4e09\u56db\u4e94\u516d\u4e03\u516b\u4e5d\u5341\u767e\u5343\u4e07\u96f6\u3007]*[\s]*|[0-9]*[\s]*)%s'

            expression_3 = re.compile("[M][o][n][a-zA-Z\s0-9:]+[0-9]")
            date_time_group_3 = expression_3.search(date_time)

            if date_time_group_1:
                date_time = date_time_group_1.group().replace('/', '-').replace('.', '-') \
                    .replace('年', '-').replace('月', '-').replace('日', '')

            elif "月" in date_time:
                try:
                    date_time = date_time.replace('年', '').replace('日', '')
                    ns = copy.deepcopy(date_time)
                    ns = ns.split()
                    for yue in range(len(ns)):
                        if '月' in ns[yue]:
                            ns[yue] = TimeFormat.chinese_to_arabic_month(ns[yue].replace('月', '').replace(',', ''))
                            ns[yue] = '%s,' % ns[yue]
                    ns = ' '.join(ns)
                    date_time = TimeFormat.chinese_to_arabic(ns)
                except Exception as E:
                    pass

            elif "昨天" in date_time.replace(' ', '').replace(' ', ''):
                ns = date_time.replace('昨天', '')
                ns = str(datetime.datetime.now())[:10] + ns
                d = TimeFormat.str_datetime(ns)
                if d:
                    t = d.timetuple()
                    time_stamp = int(time.mktime(t))
                    time_stamp = float(str(time_stamp) + str("%06d" % d.microsecond)) / 1000000 - 24 * 3600
                    date_time = TimeFormat.timestamp_datetime_str(int(time_stamp))

            elif "前天" in date_time.replace(' ', '').replace(' ', ''):
                ns = date_time.replace('前天', '')
                ns = str(datetime.datetime.now())[:10] + ns
                d = TimeFormat.str_datetime(ns)
                t = d.timetuple()
                time_stamp = int(time.mktime(t))
                time_stamp = float(str(time_stamp) + str("%06d" % d.microsecond)) / 1000000 - 2 * 24 * 3600
                date_time = TimeFormat.timestamp_datetime_str(int(time_stamp))

            elif '天前' in date_time.replace(' ', '').replace(' ', ''):
                cleaned = re.compile(number_head_base_regx % "天前")
                ns = cleaned.findall(date_time)
                if ns:
                    ns = ns[0]
                    ns = ns.replace('天前', '').replace(' ', '').replace(' ', '')
                    ns = TimeFormat.chinese_to_arabic_month(ns)
                    date_time = (datetime.datetime.now() - datetime.timedelta(days=int(ns), hours=0))
                    date_time = date_time.strftime("%Y-%m-%d %H:%M:%S")

            elif '小时前' in date_time.replace(' ', '').replace(' ', ''):
                cleaned = re.compile(number_head_base_regx % "小时前")
                ns = cleaned.findall(date_time)
                if ns:
                    ns = ns[0]
                    ns = ns.replace('小时前', '').replace(' ', '').replace(' ', '')
                    ns = TimeFormat.chinese_to_arabic_month(ns)
                    date_time = (datetime.datetime.now() - datetime.timedelta(days=0, hours=int(ns)))
                    date_time = date_time.strftime("%Y-%m-%d %H:%M:%S")

            elif '分钟前' in date_time.replace(' ', '').replace(' ', ''):
                cleaned = re.compile(number_head_base_regx % "分钟前")
                ns = cleaned.findall(date_time)
                if ns:
                    ns = ns[0]
                    ns = ns.replace('分钟前', '').replace(' ', '').replace(' ', '')
                    ns = TimeFormat.chinese_to_arabic_month(ns)
                    date_time = (datetime.datetime.now() - datetime.timedelta(days=0, hours=0, minutes=int(ns)))
                    date_time = date_time.strftime("%Y-%m-%d %H:%M:%S")

            elif date_time_group_2 and not date_time_group_1:
                date_time = date_time_group_2.group()

            elif ('cst' or 'CST' in date_time) and date_time_group_3:
                cst_time = date_time_group_3.group()
                try:
                    date_time = time.strptime(cst_time, '%a %b %d %H:%M:%S CST %Y')
                    date_time = time.strftime('%Y-%m-%d %H:%M:%S', date_time)
                except Exception as E:
                    pass

            if date_time:
                data = TimeFormat.str_datetime(date_time)
                if data:
                    data = data.strftime("%Y-%m-%d %H:%M:%S")
                    return data

    @staticmethod
    def datetime_str(datetime_instance):
        return datetime_instance.strftime("%Y-%m-%d %H:%M:%S")

    @staticmethod
    def timestamp_datetime_str(timestamp):
        date = datetime.datetime.fromtimestamp(float(timestamp)).strftime('%Y-%m-%d %H:%M:%S')
        return date

    @staticmethod
    def str_datetime_timestamp(str_datetime):
        date = datetime.datetime.strptime(str_datetime, "%Y-%m-%d %H:%M:%S")
        timestamp = time.mktime(date.timetuple())
        return timestamp

    @staticmethod
    def the_other_day(num=5):
        return datetime.datetime.now() - datetime.timedelta(days=num)

    @staticmethod
    def datetime_now_str():
        now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        return now

    @staticmethod
    def pub_time(date):
        redate = re.findall('[0-9]\d*', date)
        if redate:
            redate = redate[0]
        if (not isinstance(redate, list)) and len(redate) == 8 and '-' not in date:
            date = datetime.datetime.strptime(redate, "%Y%m%d")
            return date.strftime("%Y-%m-%d")
        else:
            return date

    @staticmethod
    def push_id_generator(data, rc, r_key, logger=None):
        default = {
            'news': 1,
            'wechat': 2
        }
        job_type = data.get('job_type')
        try:
            data['id'] = int(data.get('id')) * 100 + default.get(job_type)
        except Exception as E:
            data['id'] = 0
            data['error_process'] = '推送唯一id生成错误'
            data['error_record'] = str(traceback.format_exc())
            rc.json_push(r_key, data)
            if logger:
                logger.error('推送唯一id生成错误', extra=data)
        return data


if __name__ == '__main__':
    a = TimeFormat.time_normalization('发布时间：: 20180101 f 12:00:00 ajkdfo')
    b = TimeFormat.time_normalization('2018年06月06日 发布\n 01:00:00')
    c = TimeFormat.time_normalization('2018-07-02 08:00')
    d = TimeFormat.time_normalization("Jun 28, 2018")
    e = TimeFormat.time_normalization("Friday, November 30, 2016")
    f = TimeFormat.time_normalization("10分钟前")
    g = TimeFormat.time_normalization("11小时前")
    h = TimeFormat.time_normalization("昨天")
    i = TimeFormat.time_normalization("前天")
    j = TimeFormat.time_normalization("3天前")
    k = TimeFormat.time_normalization("20 十一月 2018")
    l = TimeFormat.time_normalization("20 十月 二零一八")
    m = TimeFormat.time_normalization("19 June 2018")
    n = TimeFormat.time_normalization("APRIL 12, 2019")
    t= TimeFormat.time_normalization("2020年08月15日")
    # print(a)
    # print(b)
    # print(c)
    # print(d)
    # print(e)
    # print(f)
    # print(g)
    # print(h)
    # print(i)
    # print(j)
    # print(k)
    # print(l)
    # print(m)
    # print(n)
    print(t)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

#############################################
# File Name: setup.py
# Author: AIIT
# Mail: zhangjun_dlut@outlook.com
# Created Time:  2020-11-10 19:17:34
#############################################


from setuptools import setup, find_packages

setup(
    name="AIITNLP",
    version="0.2.0",
    keywords=("pip", "Ner", "bidding"),
    description="Ner for bidding",
    long_description="time and path tool",
    license="MIT Licence",

    url="https://github.com/choushun/AIITNLP",
    author="AIIT",
    author_email="zhangjun_dlut@outlook.com",

    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=['jieba', 'cpca']
)




import requests
import CRFPP
import json
import os
import re
import cpca
import codecs
import jieba
from time_format import TimeFormat

pool = None
basedir = os.path.abspath(os.path.dirname(__file__))


unitAndItem_model_dir = basedir+"/model/model_unitAndItem"
loc_model_dir = basedir+"/model/model_loc"
text_model_dir = basedir+"/model/model_text"


class LocDict:
    def __init__(self):
        self.file_dict = "dict.txt"
        self.wdict = {}

    def get_dict(self):
        f = codecs.open(self.file_dict, 'r', encoding='utf-8')
        for lineno, line in enumerate(f, 1):
            line = line.strip()
            line = line.split('\t')
            w = line[0]
            if w:
                if w[0] in self.wdict:
                    value = self.wdict[w[0]]
                    value.append(w)
                    self.wdict[w[0]] = value
                else:
                    self.wdict[w[0]] = [w]

    def fmm(self, sentence):
        n = len(sentence)
        k = 0
        result = []
        while k < n:
            w = sentence[k]
            max_len = 1
            if w in self.wdict:
                words = self.wdict[w]
                t = ''
                for item in words:
                    item_len = len(item)
                    if sentence[k:k + item_len] == item and item_len >= max_len:
                        t = item
                        max_len = item_len
                if t and t not in result:
                    for i in range(k, k+max_len):
                        result.append(i)
            k = k + max_len
        return result


class BiddingExtract(object):
    """加载模型，并从文本和标题中提取结果

    Attributes:
        unitAndItem_tagger: 从标题中提取招标单位和招标物品的模型的tagger
        loc_tagger: 从标题中提取招标地点的模型的tagger
        text_tagger: 从文本中提取各类标签的模型的tagger
        model_type: 当前对象使用的哪个模型

    """
    def __init__(self, model_type):
        self.unitAndItem_tagger = CRFPP.Tagger("-m " + unitAndItem_model_dir)
        self.loc_tagger = CRFPP.Tagger("-m " + loc_model_dir)
        self.text_tagger = CRFPP.Tagger("-m " + text_model_dir)
        self.model_type = model_type
        self.dict = LocDict()

    def parse_multi_feature(self, content):
        tagger = self.loc_tagger
        fun_format = self.text_format
        self.dict.get_dict()
        loc_indexes = self.dict.fmm(content)
        tagger.clear()
        for i in range(len(content)):
            word = content[i]
            if i in loc_indexes:
                tagger.add(word + '\t' + 'Y\t')
            else:
                tagger.add(word + '\t' + 'N\t')
        tagger.parse()
        size = tagger.size()
        xsize = tagger.xsize()
        ner_tag = []
        label = ''
        words = ''
        for i in range(0, size):
            for j in range(0, xsize-1):
                char = tagger.x(i, j)
                tag = tagger.y2(i)
                if tag[0] == 'B':
                    label = re.sub('B_', '', tag)
                    words = char
                elif tag[0] == 'M':
                    words += char
                elif tag[0] == 'E':
                    words += char
                    ner_tag.append((words, label))
                    label = ''
                    words = ''
                else:
                    continue
        res = fun_format(ner_tag)
        return res

    def parse(self, content):
        """
        将模型输出的结果转换成标准化输出

        Returns:
            dict: 标准化的输出结果
        """
        if self.model_type == "title_unitAndItem":
            tagger = self.unitAndItem_tagger
            fun_format = self.sample_format
        elif self.model_type == "title_loc":
            tagger = self.loc_tagger
            fun_format = self.text_format
        else:
            tagger = self.text_tagger
            fun_format = self.text_format
        tagger.clear()
        for word in content:
            if word:
                tagger.add(word)
        tagger.parse()
        size = tagger.size()
        xsize = tagger.xsize()
        ner_tag = []
        label = ''
        words = ''
        for i in range(0, size):
            for j in range(0, xsize):
                char = tagger.x(i, j)
                tag = tagger.y2(i)
                if tag[0] == 'B':
                    label = re.sub('B_', '', tag)
                    words = char
                elif tag[0] == 'M':
                    words += char
                elif tag[0] == 'E':
                    words += char
                    ner_tag.append((words, label))
                    label = ''
                    words = ''
                else:
                    continue
        res = fun_format(ner_tag)
        return res

    def sample_format(self, res):
        """
        将模型的结果进行简单去重，方式就是set取1
        :param res:
        :return:
        """
        dataset = dict()
        for item in res:
            v = item[0]
            k = item[1]
            if k in dataset:
                dataset[k].append(v)
            else:
                dataset[k] = [v]
        for key in dataset:
            dataset[key] = list(set(dataset[key]))[0]
        return dataset

    def text_format(self, res):
        """
        文本处理模型的格式化输出，将解析的结果进行格式化输出，包括：
        label去重，同一个标签只输出一个结果；
        地址信息只输出省和市
        """
        dataset = self.sample_format(res)

        # 将时间进行格式化
        if "bidding_start_time" in dataset:
            dataset['bidding_start_time']=TimeFormat.time_normalization(dataset['bidding_start_time'])

        if "bidding_deadline" in dataset:
            dataset['bidding_deadline']=TimeFormat.time_normalization(dataset['bidding_deadline'])

        # 对金额进行格式化
        if 'bidding_budget' in dataset:
            dataset['bidding_budget']=self.money_format(dataset['bidding_budget'])
        if 'bidded_money' in dataset:
            dataset['bidded_money']=self.money_format(dataset['bidded_money'])

        # 对招标编号进行格式化
        if 'bidding_no' in dataset:
            dataset['bidding_no'] = dataset['bidding_no'].strip('：').strip('号').strip(':')

        return dataset

    def money_format(self, my):
        """
        将识别的金额进行格式化转换，转成以元为单位
        :param my:
        :return:
        """
        re_num = re.compile('\d+(\.\d+)?')
        try:
            if '亿万元' in my or '亿万' in my:
                my = re_num.search(my).group()
                my = float(my)*100000000
            elif '千万元' in my or '千万' in my:
                my = re_num.search(my).group()
                my = float(my)*10000000
            elif '百万元' in my or '百万' in my:
                my = re_num.search(my).group()
                my = float(my)*1000000
            elif '万元' in my or '万' in my:
                my = re_num.search(my).group()
                my = float(my)*10000
            else:
                my = re_num.search(my).group()
                my = float(my)
        except:
            my = None
        return my


class ZbNer(object):
    """分别得出文本和标题模型的结果，并融合

        Attributes:
            text_model: 文本模型，从text中提取实体
            title_unitAndItem_model: 从标题中提取招标单位和招标物
            title_loc_model： 从标题中提取招标地点

    """
    def __init__(self, text_model, title_loc_model, title_unitAndItem_model):
        self.text_model = text_model
        self.title_unitAndItem_model = title_unitAndItem_model
        self.title_loc_model = title_loc_model

    def get_loc(self, address):
        """
        调用一个网络接口进行地址解析，得到省和市的信息；

        Args:
            address(str): 从语料中提取的位置信息

        Returns:
            dict: {"province":"xxx", "city":"xxx"}
        """
        cpca_text = [address.strip()]
        df = cpca.transform(cpca_text)
        province = df.loc[0, "省"]
        city = df.loc[0, "市"]
        data = dict()
        data["province"] = province
        data["city"] = city
        return data

    # def loc_format(self, data):
    #     """对地址省、市进行标准化输出
    #     把文本中抽到的所有address先通过loc_resolution方法转换成xx省xx市， 然后再选用频率最高的省市作为结果，并标准化输出
    #
    #     Args:
    #         data (list(str)): 从文本中抽出的所有address
    #
    #     Returns:
    #         dict: {'province': province, 'city': city}
    #
    #     """
    #     tmp = dict()
    #     for address in data:
    #         loc_res = self.loc_resolution(address)
    #         if loc_res:
    #             province = loc_res['province']
    #             city = loc_res['city']
    #             key = (province, city)
    #             tmp[key] = tmp.get(key, 0)+1
    #     if tmp:
    #         tmp = sorted(tmp.items(), key=lambda x: x[1], reverse=True)
    #         province = tmp[0][0][0]
    #         city = tmp[0][0][1]
    #         return {'province': province, 'city': city}
    #     else:
    #         return None

    def raw_text_res(self, text):
        # 将位置字段替换为“location”，方便以后处理
        text = re.sub('\s+', u'Ж', text)
        text_res = self.text_model.parse(text)
        if "bidding_address" in text_res:
            text_res["location"] = text_res["bidding_address"]
            text_res.pop("bidding_address")
        return text_res

    def raw_title_res(self, title):
        # 合并loc和unit模型结果，将位置字段替换为“location”，方便以后处理
        title_loc_res = self.title_loc_model.parse_multi_feature(title)
        title_unitAndItem_res = self.title_unitAndItem_model.parse(title)
        if "bidding_location\r" in title_loc_res:
            title_loc_res["location"] = title_loc_res["bidding_location\r"]
            title_loc_res.pop("bidding_location\r")
        title_loc_res.update(title_unitAndItem_res)
        return title_loc_res

    def standard_res(self, raw_res):
        # 返回经过地址解析的结果
        standard_res = raw_res
        if "location" in raw_res:
            standard_res["location"] = self.get_loc(raw_res["location"])
        return standard_res

    def fuse(self, text, title):
        text_res = self.raw_text_res(text)
        title_res = self.raw_title_res(title)
        # 大部分都以文本中提取内容为主
        fuse_res = text_res
        fuse_res['location'] = None
        if "location" in title_res:
            fuse_res["location"] = self.get_loc(title_res['location'])
        else:
            if 'bidding_address' in text_res:
                fuse_res['location'] = self.get_loc(text_res['bidding_address'])
        if "bidding_unit" in title_res and "bidding_unit" not in text_res:
            fuse_res["bidding_unit"] = title_res.get("bidding_unit")
        if "bidding_item" in title_res:
            fuse_res["bidding_product"] = title_res.get("bidding_item")
        return fuse_res

    def cut(self, title_only=False, text_only=False, **corpus):
        # 默认为标题和正文抽取，调用格式为:cut(text=xxx, title=xxx)
        # 若只抽标题，调用格式为:cut(title_only=True, title=xxx)
        # 若只抽正文，调用格式为:cut(text_only=True, text=xxx)
        if title_only:
            raw_res = self.raw_title_res(corpus['title'])
            return self.standard_res(raw_res)
        elif text_only:
            raw_res = self.raw_text_res(corpus['text'])
            return self.standard_res(raw_res)
        else:
            return self.fuse(corpus['text'], corpus['title'])


# 分别实例化文本抽取，标题位置抽取和标题公司招标物抽取对象
# 传入融合模型类中，进行操作
BEText = BiddingExtract("text")
BETitle_loc = BiddingExtract("title_loc")
BETitle_unitAndItem = BiddingExtract("title_unitAndItem")
FM = ZbNer(BEText, BETitle_loc, BETitle_unitAndItem)
zb_ner_extract = FM.cut


def _cut(text, title):
    return FM.cut(text=text, title=title)


def _cut_text_only(text):
    return FM.cut(text_only=True, text=text)


def _cut_title_only(title):
    return FM.cut(title_only=True, title=title)


def enable_parallel(processnum=None):
    global pool, zb_ner_extract, FM
    from multiprocessing import cpu_count
    if os.name == 'nt':
        raise NotImplementedError(
            "jieba: parallel mode only supports posix system")
    else:
        from multiprocessing import Pool
    if processnum is None:
        processnum = cpu_count()
    pool = Pool(processnum)
    zb_ner_extract = _pcut


def _pcut(text_only=False, title_only=False, **corpus):
    if text_only:
        result = pool.map(_cut_text_only, corpus['text'])
    elif title_only:
        result = pool.map(_cut_title_only, corpus['title'])
    else:
        result = pool.starmap(_cut, zip(corpus['text'], corpus['title']))
    return result


def test():
    with open("data/test_title.txt", 'r', encoding="utf-8") as titles:
        test_title = titles.readlines()
    with open("data/test_text.txt", 'r', encoding="utf-8") as texts:
        test_text = texts.readlines()
    test_title = "华润雪花啤酒(安徽)有限公司六安工厂ЖD线增加标准瓶检测回流带、A线卸罐垛机垛位公告"
    test_text = "华润雪花啤酒(安徽)有限公司六安工厂ЖD线增加标准瓶检测回流带、A线卸罐垛机垛位公告Ж寻源公告公告编号:XHXYGG202009090103一、采购项目基本情况采购人:华润雪花啤酒(安徽)有限公司采购项目编号:XHCGXY202009090103采购项目名称:六安工厂ЖD线增加标准瓶检测回流带、A线卸罐垛机垛位采购内容和范围:1)Ж包装A线卸罐垛机新增垛位Ж负责在包装A线卸罐垛机的上垛区始端增加1个转向输垛机,在转向输送垛机后增加4个1.6米链条垛板输送机;Ж负责提供配套使用的变频器、光电等电器元件的提供,详见附件1.3:报价单;1.4:设备平面布置图;Ж负责上述设备的运输、安装、调试及售后服务。2)Ж包装D线增加标准瓶检测回流带Ж负责包装D线验瓶机后增加标准瓶检测回流带,包括:输瓶机、磁性弯头、防尘罩、减速机等,详见1.3:报价单、1.4:设备平面布置图;负责上述设备的运输、安装、调试及售后服务。质保期:不低于一年。二、供应商资格要求Ж资审方式:资格后审供应商必须具备的资格和要求:三证合一,具备机械设备加工能力三、采购文件的获取采购文件在华润集团守正电子招标平台发布,不再另行线下提供纸质采购文件,凡有意参与者可在本公告期间自行登陆守正平台查看和下载采购文件。四、响应文件的提交响应文件提交/报价截止时间:Ж2020-09-17Ж12:00:00Ж(北京时间,若有变化另行通知)。响应文件提交/报价方式:在响应文件提交/报价截止时间前,通过华润集团守正电子招标平台提交电子响应文件或报价,逾期提交将被拒收,五、采购人联系方式Ж联系人:刘婷婷电话:13003058989邮箱:liutingting2@crb.cn六、采购明细行号物品/项目名称单位需求数量补充说明1D线增加标准瓶检测回流带、A线卸罐垛机垛位套2具体详见附件七、采购说明具体内容详见附件Ж八、其它事项1.本公告在华润集团守正电子招标平台(https://szecp.crc.com.cn)上公开发布。2.本项目采购通过守正平台线上进行,供应商需注册华润集团守正电子招标平台,按需办理CA电子钥匙(用于需插CA锁报价的项目),通过平台进行响应文件的递交或报价,具体操作步骤可查阅网站首页帮助中心的操作手册,也可以联系守正客服。3.答疑澄清、通知等文件一经在华润集团守正电子招标平台发布,视为已发放给相应供应商(发放时间即为发出时间),请随时关注华润集团守正电子招标平台发布的相关信息,并及时查阅和处理。"
    res = zb_ner_extract(text=test_text, title=test_title)
    print(res)


if __name__ == "__main__":
    test()
